"""
Tool Name: Grid Label Creation 
Source Name: grid.py
Version: ArcGIS 10 --- v.3
Author: Giovanni Zambotti --- CGA Harvard (gzambotti@cga.harvard.edu)
Date - 2/14/2011
Purpose -  Creating grid labels to use for printing maps
"""
# Import system modules
import arcpy, os
from arcpy import env

# Set the workspace and user input parameters

inFeatures = arcpy.GetParameterAsText(0)
tileName = arcpy.GetParameterAsText(1) 

# overright layers
arcpy.env.overwriteOutput = True

# set local parameters
inLyr = "lyr"
inLyr1 = "lyr1"
inLyr2 = "lyr2"
inLyr3 = "lyr3"
inLyr4 = "lyr4"
inLyr5 = "lyr5"
fieldNameID = "ID"

expression2 = "\"\""
fieldLength = 50

northTile = "North_Tile"
southTile = "South_Tile"
eastTile = "East_Tile"
westTile = "West_Tile"

outTile = "tile"
outFeature = "point"


# Put in error trapping in case an error occurs when running tool
try:
   
    # add a new field to the inFeatures file
    arcpy.AddField_management(inFeatures, fieldNameID, "LONG", "", "", "", "ID", "NULLABLE")    

    fieldList = arcpy.ListFields(inFeatures, "", "OID")

    for field in fieldList:
        if field.name == 'OBJECTID':
            arcpy.CalculateField_management(inFeatures, fieldNameID, "\"!" + field.name + "!\"", "PYTHON")
            queryField = field.name
        else:
            arcpy.CalculateField_management(inFeatures, fieldNameID, "\"!" + field.name + "!\"", "PYTHON")
            queryField = field.name    

    # add field for directions
    arcpy.AddField_management(inFeatures, northTile, "TEXT", "", fieldLength)
    arcpy.AddField_management(inFeatures, southTile, "TEXT", "", fieldLength)
    arcpy.AddField_management(inFeatures, eastTile, "TEXT", "", fieldLength)
    arcpy.AddField_management(inFeatures, westTile, "TEXT", "", fieldLength)   
    
    arcpy.CalculateField_management(inFeatures, northTile, expression2, "PYTHON")
    arcpy.CalculateField_management(inFeatures, southTile, expression2, "PYTHON")
    arcpy.CalculateField_management(inFeatures, eastTile, expression2, "PYTHON")
    arcpy.CalculateField_management(inFeatures, westTile, expression2, "PYTHON")
    
    # Make a layer from the feature class
    arcpy.MakeFeatureLayer_management(inFeatures, inLyr)
    
    # start looping into each record
    rows = arcpy.UpdateCursor(inLyr)
    x = 0
    row = rows.next() 

    while row:            
        x = x + 1

        tileCount = len(inLyr) + x
        arcpy.SetProgressor("step", "Processing ...", 0, tileCount, 1)
        
        q = "\"" + str(queryField) + "\" = "  + str(x)
        q1 = "\"ID\" <> "  + str(x)                
        # create a tile and save it
        arcpy.SelectLayerByAttribute_management(inLyr, "NEW_SELECTION", q)        
        # saved out the selected tile
        arcpy.CopyFeatures_management(inLyr, outTile)   

        arcpy.MakeFeatureLayer_management(inFeatures, inLyr4)
        arcpy.MakeFeatureLayer_management(outTile, inLyr1)        
        arcpy.SelectLayerByLocation_management(inLyr4, "intersect", inLyr1, "50 Feet", "NEW_SELECTION")
        outfoo = "tfoo"        
        arcpy.CopyFeatures_management(inLyr4, outfoo)
        
        arcpy.MakeFeatureLayer_management(outfoo, inLyr5)
        arcpy.SelectLayerByAttribute_management(inLyr5, "NEW_SELECTION", q1)
        # make feature point from polygon tiles and saved as points
        arcpy.FeatureToPoint_management(inLyr5, inLyr2, "INSIDE")        
        arcpy.CopyFeatures_management(inLyr2, outFeature)
        
        arcpy.MakeFeatureLayer_management(outFeature, inLyr3)

        # looping inside the point layer
        prows = arcpy.SearchCursor(inLyr3)
        z = 0
        prow = prows.next()

        #for prow in prows:
        while prow:
            z = z + 1
            q2 = "\"" + queryField + "\" =" + str(z)
            
            
            arcpy.SelectLayerByAttribute_management(inLyr3, "NEW_SELECTION", q2)
            value1 = prow.getValue(tileName)
            arcpy.Near_analysis(outTile, inLyr3, "", "", True)
            
            # run the near command and update the table
            trows = arcpy.SearchCursor(outTile)            
            trow = trows.next()
            arcpy.SetProgressorLabel("Labeling Tile " + str(x) + " ...")
            arcpy.SetProgressorPosition()
            while trow:
                arcpy.Near_analysis(outTile, inLyr3, "", "", True)
                # read the angle update the direction field                
                value2 = trow.getValue("NEAR_ANGLE")

                if value2 >= -5 and value2 <= 5:
                    row.East_Tile = str(value1)                    
                    rows.updateRow(row)
                elif value2 >= 85 and value2 <= 95:
                    row.North_Tile = str(value1)                    
                    rows.updateRow(row)
                elif value2 >= 175 or value2 <= -175:
                    row.West_Tile = str(value1)                    
                    rows.updateRow(row)
                elif value2 >= -95 and value2 <= -85:
                    row.South_Tile = str(value1)                    
                    rows.updateRow(row)
                
                trow = trows.next()
				
            prow = prows.next()
        
        del prows, prow 
        

        row = rows.next()
        #arcpy.ResetProgressor()
    del rows, row

      
except Exception, e:
    # If an error occurred, print line number and error message  
    import traceback, sys
    tb = sys.exc_info()[2]
    print "Line %i" % tb.tb_lineno
    print e.message

